﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SBOTestApplication
{
	public partial class FormSBOTest : Form
	{
		public FormSBOTest()
		{
			InitializeComponent();
		}

		private void DoPartialPicking(SAPbobsCOM.Company company, SAPbobsCOM.PickLists picklist, int linenum, string batchnumber, double quantity)
		{
			string s = picklist.GetAsXML();
			picklist.SaveXML("picklist_original.xml");
			lstLog.Items.Add("Picklist loaded. It is saved as xml to picklist_original.xml");
			picklist.Lines.SetCurrentLine(linenum);
			picklist.Lines.PickedQuantity = quantity;

			picklist.Lines.BatchNumbers.BatchNumber = batchnumber;
			picklist.Lines.BatchNumbers.Quantity = quantity;
			picklist.Lines.BatchNumbers.Add();

			s = picklist.GetAsXML();
			picklist.SaveXML("picklist_beforeupdate.xml");
			lstLog.Items.Add("Picklist modified. Before update it is saved as xml to picklist_beforeupdate.xml");

			int returnCode = picklist.Update();

			if (returnCode != 0)
			{
				string errorMessage = company.GetLastErrorDescription();

				throw new Exception(returnCode.ToString() + " - " + errorMessage + " - " + "Batch picking failed.");
			}
			else
			{
				LogAndShowMessage("Batch picking successful.");
			}

		}

		private SAPbobsCOM.Company GetConnection(string connectionString)
		{
			string[] csArray = connectionString.Split(';');

			Dictionary<string, string> cs = new Dictionary<string, string>();

			foreach (string s in csArray)
			{
				string[] sArr = s.Split('=');

				cs.Add(sArr[0], sArr[1]);
			}

			SAPbobsCOM.Company company = new SAPbobsCOM.Company();

			company.Server = cs["Server"];
			company.CompanyDB = cs["CompanyDB"];
			company.UseTrusted = StringComparer.InvariantCultureIgnoreCase.Compare(cs["UseTrusted"], "True") == 0;
			company.DbUserName = cs["DbUserName"];
			company.DbPassword = cs["DbPassword"];
			company.UserName = cs["UserName"];
			company.Password = cs["Password"];
			company.DbServerType = (SAPbobsCOM.BoDataServerTypes)Enum.Parse(typeof(SAPbobsCOM.BoDataServerTypes), "dst_" + cs["ServerType"]);
			company.LicenseServer = cs["LicenseServer"];

			if (company.Connect() != 0)
			{
				int errorCode = company.GetLastErrorCode();

				string errorMessage = company.GetLastErrorDescription();

				throw new Exception(errorCode.ToString() + " - " + errorMessage + " - " + string.Format(" Connect failed to company db '{0}' on server '{1}' of SBO", cs["CompanyDB"], cs["Server"]));
			}

			return company;
		}

		private void LogAndShowMessage(string message)
		{
			System.Diagnostics.Debug.WriteLine(message);

			lstLog.Items.Add(message);

			MessageBox.Show(message);
		}

		private void btnPickBatch_Click(object sender, EventArgs e)
		{
			SAPbobsCOM.Company company = null;
			SAPbobsCOM.PickLists picklist = null;

			try
			{
				System.Configuration.ConnectionStringSettings css = System.Configuration.ConfigurationManager.ConnectionStrings["SboConnectionString"];

				company = GetConnection(css.ConnectionString);

				picklist = company.GetBusinessObject(SAPbobsCOM.BoObjectTypes.oPickLists);
				picklist.GetByKey(int.Parse(txtPickListDocEntry.Text));

				DoPartialPicking(company, picklist, int.Parse(txtPickListLineNum.Text), txtBatchNumber.Text, double.Parse(txtQuantity.Text));
			}
			catch (Exception ex)
			{
				LogAndShowMessage(ex.Message);
			}
			finally
			{
				try
				{
					System.Runtime.InteropServices.Marshal.FinalReleaseComObject(picklist);
					picklist = null;
				}
				catch { }

				try
				{
					if (company.Connected)
					{
						company.Disconnect();
					}

					System.Runtime.InteropServices.Marshal.FinalReleaseComObject(company);
					company = null;
				}
				catch { }
			}
		}

		private void lblSalesOrder1_Click(object sender, EventArgs e)
		{

		}
	}
}
